pub(crate) enum SequentialFocusNavigationMechanism {
Dom,
Sequential(i32),
FirstOrLast,
}Expand description
https://html.spec.whatwg.org/multipage/#selection-mechanism
This also incorporates the case where the starting point is a navigable as that is a distinct set of behaviors from the two kinds of mechanisms listed in the specification.
Variants
Dom
Sequential(i32)
FirstOrLast
This case isn’t mentioned explicitly in the specification, but it’s implied. It works
like Sequential, but without a starting point. This kind of search will return the
first or last (depending on direction) sequentially focusable element in sequential
focus order. This is used in two situations:
- When the starting point is a navigable
- When descending into a nested focus scope