pub(crate) enum SequentialFocusNavigationSearchContext {
Original,
Nested,
Containing,
}Variants
Original
The focus scope that initiated this search. Containing contexts and nested contexts can both be searched.
Nested
The search has descended into a nested search scope. Containing contexts should never be searched.
Containing
The search has ascended to search a containing search scope. The starting point’s focus scope should never be searched to avoid cycles.